#!/usr/bin/env python3
"""Creates Module Glue Weight Json from input file and upload."""
import argparse
import json
import sys
from copy import deepcopy
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils, UploadMultipleTests
HELP_LINK="https://itkdb-gtk.docs.cern.ch"


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


def str2bool(v):
    """Boolean from string."""
    return v.lower() in ("yes", "true", "t", "1")


def find_module(ifile, module_sn="MODULE_SN"):
    """Find the starting line of a module.

    Args:
        ifile: the input file object.
        module_sn: tag t ostar t a new module.

    Returns:
        the SN of the module.

    """
    line = ifile.readline()
    if line is None:
        return None

    while line:
        if module_sn in line:
            data = [x.strip() for x in line.split()]
            if len(data) != 2:
                print("Inconsisten data line should be: MODULE_SN serial_number")

            if len(data) >= 2:
                return data[1]
            else:
                line = ifile.readline()

        else:
            line = ifile.readline()

    return None


def remove_defaul_keys(data, default_value=-9999):
    """Remove keys with deafault value.

    Args:
        data: Input dictionary
        default_value: Defaultvalue. Defaults to -9999.

    Returns:
        data: trimmed dictionary

    """
    keys = []
    for key, val in data.items():
        if val == default_value:
            keys.append(key)

    for key in keys:
        del data[key]

    return data


class GlueWeight(dbGtkUtils.ITkDBWindow):
    """Upluead Glue Weight test."""

    def __init__(self, session, ifile=None, help_link=None):
        """Initialization.

        Args:
            session: ITkDB session_
            ifile (optional): Input file. Defaults to None.

        """
        self.ifile = ifile
        self.session = session
        self.modules = []
        self.test_list = []
        self.test_index = {}

        defaults = {
            "institution": "IFIC",
            "runNumber": "1",
            "properties": {
                "START_DATE": ITkDButils.get_db_date(),
                "FINISH_DATE": ITkDButils.get_db_date()
            }
        }
        self.skeleton = ITkDButils.get_test_skeleton(
            session, "MODULE", "GLUE_WEIGHT", defaults)

        global gtk_runs
        if gtk_runs:
            super().__init__(session=session,
                             title="Upload Glue Weight",
                             help_link=help_link)
            self.init_window()
        else:
            self.pdb_user = ITkDButils.get_db_user(session)

    def init_window(self):
        """Initialize window."""
        #
        # Prepare HeaderBar
        self.set_border_width(10)

        # Prepare HeaderBar
        self.hb.props.title = "Glue height"

        # file entry and search button
        self.file_name = Gtk.FileChooserButton()
        self.file_name.connect("file-set", self.on_file_set)
        self.mainBox.pack_start(self.file_name, False, True, 0)
        if self.ifile and self.ifile.name:
            self.file_name.set_filename(self.ifile.name)

        # Add a Separator
        self.mainBox.pack_start(Gtk.Separator(orientation=Gtk.Orientation.VERTICAL), False, True, 0)

        # The notebook
        self.notebook = Gtk.Notebook()
        self.notebook.set_tab_pos(Gtk.PositionType.LEFT)
        self.mainBox.pack_start(self.notebook, True, True, 20)
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.set_border_width(5)

        label = Gtk.Label()
        label.set_markup("<big>You should select a file using the file selector above</big>")
        label.set_line_wrap(True)
        box.pack_end(label, True, True, 0)
        self.notebook.append_page(box)

        if self.ifile:
            self.modules = self.parse_file(self.ifile)
            self.populate_notebook()



        # The text view
        self.mainBox.pack_end(self.message_panel.frame, True, True, 5)

        # The button box
        btnBox = Gtk.ButtonBox(orientation=Gtk.Orientation.HORIZONTAL)
        self.buttons = []

        btn = Gtk.Button(label="Reload file")
        btn.connect("clicked", self.on_read_file)
        btnBox.add(btn)

        btn = Gtk.Button(label="Upload Tests")
        btn.connect("clicked", self.on_upload)
        if self.ifile is None or self.ifile.name is None:
            btn.set_sensitive(False)

        self.buttons.append(btn)
        btnBox.add(btn)

        self.mainBox.pack_start(btnBox, False, True, 0)

        self.show_all()

    def create_test_window(self, test_json, test_name, label):
        """Create the dialog for a DB test and add it to the notebook.

        Args:
            test_json: The JSon-like dict with the values
            test_name: The name of the test for internal indexing
            label: The label for the Notebook

        Returns:
            The box containing the data.

        """
        scrolled, gM = dbGtkUtils.create_scrolled_dictdialog(test_json)
        self.test_list.append(gM)
        self.test_index[test_name] = len(self.test_list) - 1
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.set_border_width(5)
        box.pack_end(scrolled, True, True, 0)
        box.dict_dialog = gM
        gM.box = box
        self.notebook.append_page(box, Gtk.Label(label=label))

        return box

    def populate_notebook(self):
        """Populate noebook."""
        dbGtkUtils.empty_container(self.notebook)
        self.test_list = []
        self.test_index = {}

        for m in self.modules:
            self.create_test_window(m, m["component"], m["component"])

        self.notebook.show_all()

    def on_file_set(self, *args):
        """Call when file chosen in FileChooser."""
        self.on_read_file(None)

    def on_upload(self, *args):
        """Upload tests to DB."""
        self.upload_tests()

    def on_read_file(self, widget):
        """Read file button clicked."""
        path = self.file_name.get_filename()
        if path is None or not Path(path).exists():
            dbGtkUtils.complain("Could not find Production File", path, parent=self)
            return

        self.read_file(path)
        self.populate_notebook()
        for btn in self.buttons:
            btn.set_sensitive(True)

    def upload_current_test(self, *args):
        """Upload test i ncurrent window."""
        pass

    def read_file(self, path):
        """Parse file."""
        with open(path, "r", encoding="UTF-8") as ifile:
            self.modules = self.parse_file(ifile)

    def parse_file(self, ifile, module_sep="MODULE_SN"):
        """Parse a data file.

        Args:
            ifile (): A file object.
            module_sep: tag to idenntify a new module.

        Returns:
            a list of tests (json)

        """
        ifile.seek(0, 0)

        # Now get the modules in the file
        modules = []
        while True:
            SN = find_module(ifile, module_sn=module_sep)
            if SN is None:
                break

            modules.append((SN, ifile.tell()))

        # Read the module data block
        module_data = []
        for SN, ipos in modules:
            ifile.seek(ipos, 0)

            line = ifile.readline()
            test_json = deepcopy(self.skeleton)
            test_json["component"] = SN
            while line:
                if module_sep in line:
                    break

                if line[0] == '#':
                    line = ifile.readline()
                    continue

                data = [x.strip() for x in line.split()]
                if len(data) < 2:
                    line = ifile.readline()
                    continue

                if data[0] in test_json["properties"]:
                    if isinstance(test_json["properties"][data[0]], str):
                        test_json["properties"][data[0]] = data[1]
                    else:
                        test_json["properties"][data[0]] = float(data[1])

                elif data[0] in test_json["results"]:
                    test_json["results"][data[0]] = float(data[1])

                else:
                    key = data[0].lower()
                    if key == "comments":
                        txt = " ".join(data[1:]).split(';')
                        comments = [x.strip() for x in txt]
                        test_json["comments"] = comments

                    if key == "defects":
                        txt = " ".join(data[1:]).split(':')
                        defect = [x.strip() for x in txt]
                        test_json["defects"].append({
                            "name": defect[0],
                            "description": defect[1],
                            "properties": {}
                        })

                    else:
                        if key == "passed":
                            test_json["passed"] = str2bool(data[1])

                        elif key == "problems":
                            test_json["problems"] = str2bool(data[1])

                line = ifile.readline()

            # clean up
            # remove_defaul_keys(test_json["properties"])
            remove_defaul_keys(test_json["results"])
            module_data.append(test_json)

        return module_data

    def upload_tests(self):
        """Upload tests to DB."""
        W = UploadMultipleTests.UploadMultipleTests(
            self.session,
            help_link="{}/uploadMultipleTests.html".format(HELP_LINK)
        )
        
        if len(self.test_list)>0:
            for G in self.test_list:
                m = G.values
                W.add_test_data_to_view(m)
                

        else:
            for m in self.modules:
                W.add_test_data_to_view(m)


def main():
    """Main entry."""
    # Get input file
    parser = argparse.ArgumentParser(description="GlueTest")
    parser.add_argument('files', nargs='*', help="Input files")
    parser.add_argument("--separator", default="MODULE_SN", help="Key for module serial number.")
    options = parser.parse_args()

    if not gtk_runs:
        if len(options.files) == 0:
            print("I need an input file")
            sys.exit(1)

        try:
            ifile = open(options.files[0], 'r')

        except Exception as e:
            print(e)
            sys.exit()

    else:
        try:
            ifile = open(options.files[0], 'r') if len(options.files)>0 else None

        except Exception as e:
            print(e)
            ifile = None

    # Connect to DB and get the test skeleton
    dlg = ITkDBlogin.ITkDBlogin()
    session = dlg.get_client()
    if session is None:
        print("Could not connect to DB with provided credentials.")
        sys.exit()

    GW = GlueWeight(session, ifile, help_link=HELP_LINK)
    if gtk_runs:
        GW.show_all()
        GW.set_accept_focus(True)
        GW.present()
        GW.connect("destroy", Gtk.main_quit)

        try:
            Gtk.main()

        except KeyboardInterrupt:
            print("Arrrgggg!!!")

    else:
        GW.modules = GW.parse_file(ifile)
        print(json.dumps(GW.modules, indent=2))
        GW.upload_tests()
        ifile.close()

    dlg.die()


if __name__ == "__main__":
    main()



import random

# initialize
random.seed()

# module_param[iring][i_sensor][i_hybrid][i_row]
module_param = { 
    0: [
        [
            [[255, 830], [1343, 1918], [2431, 3006], [3519, 4094]],
            [[831, 1342], [1919, 2430], [3007, 3518], [4095, 4606]],
        ]
    ],
    1: [
        [
            [[271, 974], [1615, 2318], [2959, 3662], [4303, 5006]],
            [[975, 1614], [2319, 2958], [3663, 4302], [5007, 5646]],
        ]
    ],
    2: [
        [
            [[201, 968], [969, 1736], [1737, 2504], [2505, 3272]]
        ]
    ],
    3: [
        [
            [[566, 1013], [2358, 2805], [4150, 4597], [5942, 6389]],
            [[1462, 1909], [3254, 3701], [5046, 5493], [6838, 7285]]
        ],
        [
            [[1014, 1461], [2806, 3253], [4598, 5045], [6390, 6837]],
            [[1910, 2357], [3702, 4149], [5494, 5941], [7286, 7733]]
        ]
    ],
    4: [
        [
            [[318, 829], [1342, 1853], [2366, 2877], [3390, 3901]]
        ],
        [
            [[830, 1341], [1854, 2365], [2878, 3389], [3902, 4413]]
        ]
    ],
    5: [
        [
            [[332, 907], [1484, 2059], [2636, 3211], [3788, 4363]]
        ],
        [
            [[908, 1483], [2060, 2635], [3212, 3787], [4364, 4939]]
        ]
    ],
}


def wire2strip(mod_par, irow, iwire):
    """Convert from wirebond index to strip_number."""
    for isensor in mod_par:
        for ihyb in isensor:
            rng = ihyb[irow-1]
            if iwire>= rng[0] and iwire<=rng[1]:
                if irow % 2:
                    ichan = 2*(iwire-rng[0]) + 1
                else:
                    ichan = 2*(iwire-rng[0])
                    
                return ichan

    return None


def get_module_type(SN):
    """Get parameters of module type.

    Args:
        SN: Serial Number

    Returns:
        list: list with bond numbers.
    """
    if len(SN) != 14 or SN[:3]!="20U":
        raise ValueError("Wrong serial number {}".format(SN))
    
    if SN[3:5] != "SE":
        raise ValueError("Cannot handle barrel modules yet.")
    
    mod_type  = SN[5:7]
    if mod_type[0] != "M":
        raise ValueError("Does not seem to be a RingModule: {}".format(SN))

    ring = int(mod_type[-1])
    param = module_param[ring]

    return param



def find_holes(chan_list, max_chan=1e10):
    """Fidn groups of consecutive channels."""
    out = sorted(chan_list)
    nchan = 0
    last_chan = -1
    ichan = -1
    holes = []
    for chan in out:
        if last_chan < 0:
            last_chan = chan
            continue
        
        if chan - last_chan > 1:
            if nchan:
                holes.append([ichan, nchan])
            nchan = 0
            ichan = -1
        else:
            if last_chan < max_chan and chan >= max_chan:
                # WE are in another sensor
                holes.append([ichan, nchan])
                nchan = 0
                ichan = -1
                last_chan = chan
                continue
            
            nchan += 1
            if ichan < 0:
                ichan = last_chan
                nchan += 1
            
        last_chan = chan
    
    if nchan:
         holes.append([ichan, nchan])
    return holes


iring = random.randint(0, 5)
mod_par = module_param[iring]
nsensor = len(mod_par)
isensor = random.randint(0, nsensor-1)
nhyb = len(mod_par[0])
ihyb = random.randint(0, nhyb-1)

rng = mod_par[isensor][ihyb][0]
nchip = (rng[1]-rng[0]+1)/64
max_chan = 128 * nchip

segment = random.randint(0, 1)
nchan = random.randint(1, 10)
bonds = []
channels = []
rng = mod_par[isensor][ihyb]
for i in range(nchan):
    ichan = random.randint(0, max_chan-1)
    channels.append(ichan)
    if ichan % 1:
        irow = 1
    else:
        irow = 2
        
    irow += 2*segment
    ilow = rng[irow-1][0]
    ibond = int(ilow + ichan/2)
    
    jchan = wire2strip(mod_par, irow, ibond)
    bonds.append(jchan)

print(channels)
print(bonds)

SN = "20USEM00000040"
SN = "20USEM30000030"
mod_par = get_module_type(SN)
ichan = wire2strip(mod_par, 1, 334)

chan_list = [0, 3, 9, 20, 21, 22, 50, 60, 61, 62, 63, 64 ]
H = find_holes(chan_list, 62)
print(H)



"""Utilities for the inteaction with the ITkDB."""
from collections.abc import Iterable
from datetime import datetime
from pathlib import Path
import getpass

import dateutil.parser
import itkdb

# The response of the DB
db_response = ""
attachment_urls = {}
uploaded_test_runs = []


# define an Attachment object.
class Attachment(object):
    """Encapsulates Attachment information."""

    def __init__(self, path=None, url=None, title=None, desc=None):
        """Initialization."""
        if path is not None:
            self.path = Path(path).expanduser().resolve()
            self.type = "file"
        else:
            self.path = None

        if url is not None:
            self.url = url
            self.type = "link"
        else:
            self.url = None

        if self.path and self.url:
            raise ValueError("Invalid Attachment. Has both file and link.")

        self.title = title
        self.desc = desc


def is_iterable(obj):
    """Tell if an object is iterable. Strings are not considered iterables."""
    if isinstance(obj, Iterable):
        if isinstance(obj, str) or isinstance(obj, bytes):
            return False
        else:
            return True
    else:
        return False


def get_db_response():
    """Return the DB response.

    It is stores in a global variable. Trust the function if call
    right after your interaction with the DB.
    """
    return db_response


def get_db_date(timestamp=None):
    """Convert a date string into the expected DB format.

    Args:
        timestamp: A date in string format

    """
    def date2string(the_date):
        out = the_date.isoformat(timespec='milliseconds')
        if out[-1] not in ['zZ']:
            out += 'Z'

        return out
        # return the_date.strftime("%Y-%m-%dT%H:%M:%S.000Z")

    out = None
    if timestamp is None:
        out = date2string(datetime.now())
    elif isinstance(timestamp, datetime):
        out = date2string(timestamp)
    else:
        try:
            this_date = dateutil.parser.parse(timestamp)
            out = date2string(this_date)

        except (OverflowError, dateutil.parser.ParserError):
            out = ""

    return out



def get_petal_core_batches(session):
    """Get list of Petal core batches"""
    payload = {
        "filterMap": {
            "project": "S",
            "batchType": ["PETAL_CORE_BATCH_TYPE"],
        }
    }
    batch_list = session.get("listBatches", json=payload)
    rc = {}
    for B in batch_list:
        rc[B["number"] ]= B["id"]

    return rc

def set_petal_core_batch(session, petal_core):
    """Set the batch of the petal core."""
    if isinstance(petal_core["type"], str):
        tp = petal_core["type"]
    else:
        tp = petal_core["type"]["code"]

    if tp != "CORE_AVS":
        return

    batch_list = get_petal_core_batches(session)
    aliasID = petal_core["alternativeIdentifier"]
    if "PPB" in aliasID:
        batch_code = "PETAL_CORE_PREPROD"
    elif "PPC" in aliasID:
        batch_code = "PETAL_CORE_PROD"
    else:
        batch_code = "PETAL_CORE_PROTO"

    rc = session.post("addBatchComponent", json={"id":batch_list[batch_code], "component": petal_core["id"]})
    return rc


def registerPetalCore(client, SN, alias, HC_id=None, petal_core=None):
    """Register a Petal Core in the DB.

    Args:
        SN: Serial Number
        alias: The alias of the Petal
        HC_id: Comma separated list of HC identifiers.

    Returns:
        _type_: _description_

    """
    global db_response
    dto = {
        "project": 'S',
        "subproject": "SE",
        "institution": "AVS",
        "componentType": "CORE_PETAL",
        "type": "CORE_AVS",
        "serialNumber": SN,
        "properties": {'PRODUCTION_ID': alias}
    }
    if HC_id is not None:
        dto["properties"]["HC_ID"] = HC_id

    the_component = None

    try:
        if petal_core is None:
            db_response = client.post('registerComponent', json=dto)
            petal_core = db_response['component']

        else:
            # WE set the alternative ID and the HC_ID
            the_component = SN
            if petal_core["alternativeIdentifier"] is None:
                dto = {
                    "component": SN,
                    "code": "PRODUCTION_ID",
                    "value": alias
                }
                db_response = client.post("setComponentProperty", json=dto)

            if HC_id is not None:
                dto = {
                    "component": SN,
                    "code": "HC_ID",
                    "value": HC_id
                }
                db_response = client.post("setComponentProperty", json=dto)

        set_petal_core_batch(client, petal_core)
        return petal_core

    except KeyError:
        return None


def create_component_attachment(client, SN, file_path, title=None, description="", item_type="component"):
    """Create an attachment to the given component.

    Args:
        client: The DB client
        SN: The SN of the component.
        file_path: The pat to th efile to be attached.
        title: title of the file
        description: Description of the attachment
        item_type: the type of DB object to attach the information

    """
    global db_response
    db_response = None
    path = Path(file_path).expanduser().resolve()
    filetype =   itkdb.utils.get_mimetype(path, None)
    if title is None:
        title = path.name

    if item_type == "component":
        c_item = "component"
        cmmd = "createComponentAttachment"

    elif item_type == "shipment":
        c_item = "shipment"
        cmmd = "createShipmentAttachment"

    else:
        print("Unknown dB object")
        db_response = None
        return

    data = {
        c_item: SN,
        'title': title,
        'description': description,
        'type': 'file',
        'url': path.as_uri(),
    }
    # Get attachment data
    attachment = {'data': (path.name, open(path.as_posix(), 'rb'), filetype)}
    try:
        db_response = client.post(cmmd, data=data, files=attachment)

    except Exception as e:
        print("Error creating attachment\n", e)

    return db_response


def set_component_property(client, SN, the_property, value):
    """Set the value of an object property.

    Args:
        client: The DB client
        SN: The object SN
        the_property: The property name
        value: The property value

    """
    global db_response
    try:
        db_response = client.post('setComponentProperty',
                                  json={'component': SN,
                                        'code': the_property,
                                        'value': value})
        return db_response

    except Exception as e:
        print("Problems setting {} to {} in {}:\n\t{}".format(
            property, value, SN, str(e)))
        return None


def assemble_component(client, parent, child):
    """Assemble child into parent.

    Args:
        client: The DB client
        parent: The parent object or container.
        child: The child to be assembled.

    """
    global db_response
    try:
        db_response = client.post("assembleComponent",
                                  json={'parent': parent, 'child': child})
        return db_response

    except Exception as e:
        print("Problems assemblying {} into {}:\n\t{}".format(
            child, parent, str(e)))
        return None


def set_object_stage(client, SN, stage):
    """Set stage of object

    Args:
        client: DB session
        SN: Serial number
        stage: Stage

    """
    global db_response
    try:
        db_response = client.post("setComponentStage",
                                  json={"component": SN, "stage": stage})
        return db_response

    except Exception as e:
        print("Problems changing stage of {} into {}:\n\t{}".format(
            SN, stage, str(e)))
        return None


def get_DB_component(client, SN):
    """Get ta component by its serial number."""
    global db_response

    try:
        db_response = client.get("getComponent", json={"component": SN})
        return db_response

    except Exception as e:
        db_response = str(e)

    try:
        out = client.get('getComponent', json={'component': SN, "alternativeIdentifier":True})
        db_response = out
        return out

    except Exception as e:
        db_response = str(e)
        return None

def get_test_stages(client, SN, test_code, project=None, data=None):
    """Return a list of stages where this test has to be done.

    Args:
        client: The DB client
        SN: The object Serial Number or alternativeId
        test_code: test_code
        project: can be any of "S", "P", "CM", "CE". Dafaults to "S"
        json: the object data (result from get_DB_component)

    Returns:
        list of stages
    """
    if project is None:
        project = "S"

    if data is None:
        obj = get_DB_component(client, SN)
    else:
        obj = data


    test_desc = client.get("getTestTypeByCode",
                            json={"componentType": obj["componentType"]["code"],
                                "project": project,
                                "code": test_code})
    test_id = test_desc["id"]

    list_of_stages = []
    for S in obj["componentType"]["stages"]:
        if S["alternative"]:
            continue

        if S["testTypes"] is None:
            continue

        for T in S["testTypes"]:
            if T["testType"]==test_id:
                list_of_stages.append(S["code"])
                break

    return list_of_stages


def upload_test(client, data, attachments=None, check_runNumber=False):
    """Upload a test to the DB.

    Args:
        client: The DB client
        data (dict): A dictionary with all the elements of thee test.
        attachments ([Attachment]): one or more (in a list) Attachment to the test

    Return:
        resp: The response of the DB session.

    """
    global db_response
    global attachment_urls
    global uploaded_test_runs

    uploaded_test_runs = []
    attachment_urls = {}
    db_response = None

    # Check the given run_number. If already existing, give another one which
    # will try to be consecutive.
    if check_runNumber:
        # Get all the runNumbers in this test
        test_list = client.get("listTestRunsByComponent",
                               json={
                                    "filterMap":{
                                    "serialNumber": data["component"],
                                    "state": "ready",
                                    "testType":[data["testType"]]
                                    }
                                }
                               )
        runN= {}
        for T in test_list:
            runN[T["runNumber"]] = 1

        if data["runNumber"] in runN:
            # if the given runNumber is there, try to create a new one.
            #print("runNumber {} already in {} of {}".format(data["runNumber"], data["testType"], data["component"]))
            try:
                irun = int(data["runNumber"])
                for i in range(irun+1, 100):
                    newRn = "{}".format(i)
                    if newRn not in runN:
                        data["runNumber"] = newRn
                        break

            except ValueError:
                for i in range(100):
                    newRn = "{}_{}".format(data["runNumber"], i+1)
                    if newRn not in runN:
                        data["runNumber"] = newRn
                        break

    # Try to upload the test
    try:
        db_response = client.post("uploadTestRunResults", json=data)
        testRun = db_response["testRun"]["id"]
        uploaded_test_runs.append(testRun)

    except Exception as e:
        msg = "Could not upload the test:\n{}".format(str(e))
        return msg

    try:
        # Handle attachments.
        attachment_urls = {}
        if attachments is not None:
            if not isinstance(attachments, Iterable):
                attachments = (attachments)

            for att in attachments:
                data = {"testRun": testRun,
                        "title": att.title if att.title is not None else path.name,
                        "description": att.desc if att.desc is not None else path.name,
                        "type": att.type,
                        }
                if att.type == "file":
                    path = Path(att.path).expanduser().resolve()
                    if not path.exists():
                        print("File {} does not exist".format(path))
                        continue

                    data["url"] = path.as_uri()
                    filetype = itkdb.utils.get_mimetype(path.as_posix(), None)
                    attachment = {'data': (path.as_posix(), open(path.as_posix(), 'rb'), filetype)}
                else:
                    data["url"] = att.url
                    filetype = itkdb.utils.get_mimetype(att.url, None)
                    attachment = {'data':(att.url, None, "text/x-uri") }

                db_response = client.post('createTestRunAttachment',
                                          data=data,
                                          files=attachment)
                try:
                    url = db_response['url'].replace("https://eosatlas.cern.ch","https://cernbox.cern.ch/files/spaces")
                    attachment_urls[path.name] = url

                except KeyError:
                    pass


        return None

    except Exception as e:
        return "Could not upload attachment:\n{}".format(str(e))


def set_test_run_parameter(session, test_run, parameter, value):
    """Modify testRun Parameter

    Args:
        session: The ITkDB session
        test_run: ID of test run
        parameter: parameter code
        value: The new value
    """
    global db_response
    try:
        db_response = session.post("setTestRunParameter",
                                   json={"testRun": test_run, "code": parameter, "value": value})
        return None

    except Exception as E:
        return (str(E))

def create_test_run_comment(session, test_run, comments):
    """Adds a new comment in testRun."""
    global db_response
    if not isinstance(comments, Iterable):
        comments = (comments)

    try:
        db_response = session.post("createTestRunComment", json={"testRun": test_run, "comments": comments})
        return None

    except Exception as E:
        return (str(E))

def create_shipment(session, sender, recipient, items, name=None, send=False,
                    shipment_type="domestic", attachment=None, comments=None):
    """Create a chipment.

    Args:
        session : The itkdb session
        sender : The sender ID
        recipient : The recipient ID
        items : A list of SN of the items to be shipped.
        name: the name of the shipment.
        send: If true, the status of the shipment is updated to inTransit
        type (optional): Type of shipment. Defaults to "domestic".
        attachment (optional, :class:`Attachment`): :class:`Attachment` object.
        comments (optional): comments for the shipment

    """
    global db_response
    if name is None:
        name = "From {} to {}".format(sender, recipient)

    if shipment_type not in ["domestic", "intraContinental", "continental"]:
        db_response = "Wrong shipment type."
        return None

    if len(items) == 0:
        db_response = "Empty lit of items"
        return None

    the_comments = None
    if comments is not None:
        if not is_iterable(comments):
            the_comments = [comments]
        else:
            the_comments = comments

    data = {
        "name": name,
        "sender": sender,
        "recipient": recipient,
        "type": shipment_type,
        "shipmentItems": items,
    }

    if the_comments:
        data["comments"] = the_comments

    try:
        db_response = session.post("createShipment", json=data)

    except Exception as E:
        db_response = str(E)
        return None

    if not send:
        return db_response

    shipment_id = db_response["id"]
    payload = {
        "shipment": shipment_id,
        "status": "inTransit",
    }

    # The items
    items = []
    for item in db_response["shipmentItems"]:
        the_item = {"code": item['code'], "delivered": True}
        items.append(the_item)

    payload["shipmentItems"] = items

    db_response = None
    try:
        db_response = session.post("setShipmentStatus", json=payload)

    except Exception as E:
        db_response = str(E)
        return None

    if attachment:
        try:
            rc = create_component_attachment(session, shipment_id,
                                             attachment.path,
                                             attachment.title,
                                             attachment.desc,
                                             "shipment")
            db_response = rc

        except Exception:
            db_response = "Could not add the attachment."

    return db_response


def set_shipment_status(client, data):
    """Update shipment status."""
    global db_response
    try:
        db_response = client.post("setShipmentStatus", json=data)
        return db_response

    except Exception:
        return None


def from_full_test_to_test_data(full_test):
    """Conver getTest output to json needed to upload the test."""
    test = {
        "component": None,
        "testType": full_test["testType"]["code"],
        "institution": full_test["institution"]["code"],
        "runNumber": full_test["runNumber"],
        "date": full_test["date"],
        "passed": full_test["passed"],
        "problems": full_test["problems"],
        "properties": {},
        "results": {},
        "comments": [],
        "defects": []
    }

    try:
        test["component"] = full_test["components"][0]["serialNumber"]

    except Exception:
        print("from_full_test_to_test_data\nPossible error: no SN found for component.")
        pass

    for P in full_test["properties"]:
        test["properties"][P["code"]] = P['value']

    if full_test["results"]:
        for P in full_test["results"]:
            test["results"][P["code"]] = P['value']

    if full_test["comments"]:
        for C in full_test["comments"]:
            test["comments"].append(C["comment"])

    if full_test["defects"]:
        for D in full_test["defects"]:
            test["defects"].append({"name": D["name"],
                                    "description": D["description"],
                                    "properties": D["properties"]})

    return test


def get_testrun(session, test_id, out_type="object"):
    """Retrieve information about a given test.

    Args:
        session : The itkdb session
        test_id : The ID of the test.
        out_type (optional): Type of output (full or object). Defaults to "object".

    """
    global db_response
    try:
        db_response = session.get("getTestRun", json={"testRun": test_id})
        return db_response

    except Exception:
        return None


def get_test_skeleton(session, component, test_code, userdef=None, uservalues=None):
    """Get the skeleton of the given test.

    Args:
        session: The DB session
        component: The component which is tested
        test_code: The test code
        userdef: default values of test parameters, propertines and results.
        uservalues: default values for different types.

    """
    global db_response

    if userdef is None:
        userdef = {}

    if uservalues is None:
        uservalues = {}

    defvalues = {
        "string": "",
        "integer": -9999,
        "float": -9999.0,
        "boolean": True,
    }
    for ky, val in defvalues.items():
        if ky not in uservalues:
            uservalues[ky] = val

    data = {"project": "S", "componentType": component}
    out = session.get("listTestTypes", json=data)
    db_response = out
    the_test = None
    for tst in out:
        # print(tst['code'])
        if tst['code'] == test_code:
            the_test = tst
            break

    if the_test is None:
        print("test {} not found for {}".format(test_code, component))
        return None

    # for prop in the_test["properties"]:
    #     print("{}; {}".format(prop["code"], prop["name"]))

    # for prop in the_test["parameters"]:
    #     print("{}; {}".format(prop["code"], prop["name"]))

    skltn = {
        "component": None,
        "testType": test_code,
        "institution": None,
        "runNumber": "-1",
        "date": get_db_date(),
        "passed": True,
        "problems": False,
        "properties": {},
        "results": {},
        "comments": [],
        "defects": []
    }

    # Set default values
    for key, val in skltn.items():
        if key in userdef:
            if isinstance(val, dict):
                continue

            skltn[key] = userdef[key]

    def get_default(vin, default=None):
        # print(json.dumps(vin, indent=1))
        if vin['valueType'] == "single":
            vtype = vin['dataType']
            val = None
            if vtype in uservalues:
                val = uservalues[vtype]
            else:
                # print("default for data type ", vtype, " not found")
                val = None

        else:
            val = None

        return val

    # SEt the properties
    for prop in the_test['properties']:
        key = prop["code"]
        if 'properties' in userdef and key in userdef['properties']:
            skltn['properties'][key] = userdef['properties'][key]
        else:
            skltn['properties'][key] = get_default(prop)

    # SEt the parameters
    for par in the_test['parameters']:
        key = par["code"]
        if 'results' in userdef and key in userdef['results']:
            skltn['results'][key] = userdef['results'][key]
        else:
            skltn['results'][key] = get_default(par)

    return skltn

def create_client():
    """Create a Client."""
    client = itkdb.Client()
    client.user._access_code1 = getpass.getpass("Access 1: ")
    client.user._access_code2 = getpass.getpass("Access 2: ")
    client.user.authenticate()
    print("Hello {} !".format(client.user.name))
    return client

def get_db_user(client):
    """REturn PDB information of current user.

    Args:
        client (itkdb.Client): The DB client.

    """
    global db_response
    if client is None:
        return None

    try:
        db_response = client.get("getUser", json={"userIdentity": client.user.identity})
        return db_response
    except Exception:
        return None



#!/usr/bin/env python3
"""PB/Hybrid panel Visual inspection GUI.."""
import sys
import copy
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils, QRScanner
from itkdb_gtk.ShowComments import ShowComments
from itkdb_gtk.ShowDefects import ShowDefects
from itkdb_gtk.UploadTest import create_json_data_editor


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, Gio, GObject


class TestJson(GObject.Object):
    """To store test JSOn."""
    __gtype_name__ = "TestJson"

    def __init__(self, js=None):
        super().__init__()
        self.js = copy.deepcopy(js)

    def set_js(self, js):
        """SEts the dictionary"""
        self.js = copy.deepcopy(js)


class PanelVisualInspection(dbGtkUtils.ITkDBWindow):
    """PB/Hybryd panel visual inspection GUI."""
    SN, ORDER, PASSED, N_FILES, F_LIST, TEST_J, ALL = range(7)
    F_DEFECT, F_NAME, F_PATH = range(3)

    def __init__(self, session, title="PanelVisualInspection", help_link=None):
        super().__init__(title=title,
                         session=session,
                         show_search="Find object with given SN.",
                         help_link=help_link)

        self.institute = self.pdb_user["institutions"][0]["code"]
        self.global_image = None
        self.global_link = None

        # action button in header
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload ALL tests.")
        button.connect("clicked", self.upload_tests)
        self.hb.pack_end(button)

        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        self.mainBox.pack_start(grid, False, False, 5)

        irow = 0
        receiver = self.create_institute_combo(only_user=True)
        receiver.connect("changed", self.on_institute)
        receiver.set_tooltip_text("Select the Institute making the test.")
        dbGtkUtils.set_combo_iter(receiver, self.institute)

        lbl = Gtk.Label(label="Institute")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)
        grid.attach(receiver, 1, irow, 1, 1)

        irow += 1
        lbl = Gtk.Label(label="Serial Number")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        self.SN = dbGtkUtils.TextEntry(small=True)
        self.SN.connect("text_changed", self.SN_ready)
        self.SN.widget.set_tooltip_text("Enter SN of PWD or Hybrid panel.")
        grid.attach(self.SN.widget, 1, irow, 1, 1)

        self.panel_type = Gtk.Label(label="")
        grid.attach(self.panel_type, 2, irow, 1, 1)

        irow += 1
        lbl = Gtk.Label(label="Date")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        self.date = dbGtkUtils.TextEntry(small=True)
        grid.attach(self.date.widget, 1, irow, 1, 1)
        self.date.entry.set_text(ITkDButils.get_db_date())
        self.date.connect("text_changed", self.new_date)

        irow += 1
        self.fC = Gtk.FileChooserButton()
        self.fC.connect("file-set", self.on_global_image)

        lbl = Gtk.Label(label="Global Image")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)
        grid.attach(self.fC, 1, irow, 1, 1)


        # Paned object
        paned = Gtk.Paned(orientation=Gtk.Orientation.VERTICAL)
        paned.set_size_request(-1, 200)
        self.mainBox.pack_start(paned, True, True, 5)

        # the list of attachments
        tree_view = self.create_tree_view()
        paned.add1(tree_view)

        # The text view
        paned.add2(self.message_panel.frame)


        self.show_all()
        self.scanner = QRScanner.QRScanner(self.get_qrcode)

    def on_global_image(self, *args):
        """We choose the global image."""
        fnam = self.fC.get_filename()
        if fnam is None or not Path(fnam).exists():
            dbGtkUtils.complain("Could not find image", fnam, parent=self)
            return

        self.global_image = Path(fnam).expanduser().resolve()



    def on_institute(self, combo):
        """A new recipient has been chosen."""
        name = self.get_institute_from_combo(combo)
        if name:
            self.institute = name

    def new_date(self, entry, value):
        """new date given at input."""
        d = dbGtkUtils.parse_date_as_string(value)
        if d is not None:
            self.date.set_text(d)

    def create_model(self):
        """Create tree view model."""
        return Gtk.ListStore(str, int, bool, int, Gtk.ListStore, TestJson)

    def create_file_model(self):
        """Create model for file list"""
        return Gtk.ListStore(str, str, str)

    def create_tree_view(self, size=150):
        """Create the TreeView with the children."""
        model = self.create_model()
        self.tree = Gtk.TreeView(model=model)
        self.tree.connect("button-press-event", self.button_pressed)

        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.tree)
        scrolled.set_size_request(-1, size)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("SN", renderer, text=PanelVisualInspection.SN)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("position", renderer, text=PanelVisualInspection.ORDER)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererToggle()
        renderer.set_property("activatable", True)
        renderer.set_property("radio", True)
        renderer.set_padding(5, 0)

        _, y = renderer.get_alignment()
        renderer.set_alignment(0, y)
        # renderer.set_property("inconsistent", True)
        renderer.connect("toggled", self.btn_toggled)

        column = Gtk.TreeViewColumn("Passed", renderer, active=PanelVisualInspection.PASSED)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("N. Images", renderer, text=PanelVisualInspection.N_FILES)
        self.tree.append_column(column)

        return scrolled

    def btn_toggled(self, renderer, path, *args):
        """Toggled."""
        model = self.tree.get_model()
        val = not model[path][PanelVisualInspection.PASSED]
        model[path][PanelVisualInspection.PASSED] = val
        model[path][PanelVisualInspection.TEST_J].js["passed"] = val

    def get_iter_at_position(self, tree, event):
        """Get the model and iterator at position."""
        # Create popup menu
        select = tree.get_selection()
        model, lv_iter = select.get_selected()
        values = None
        if lv_iter:
            values = model[lv_iter]

        else:
            P = tree.get_path_at_pos(event.x, event.y)
            if P:
                lv_iter = model.get_iter(P[0])
                values = model[lv_iter]

        return model, lv_iter, values


    def button_pressed(self, tree, event):
        """Button pressed on tree view."""
        # Create popup menu
        model, lv_iter, values = self.get_iter_at_position(tree, event)
        if not values:
            return

        # double click shows attachments
        if event.button == 1 and event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            #self.write_message("This is a double click.\n")
            self.on_upload_image(None, (model, lv_iter, values))
            return

        if event.button != 3:
            return

        menu = Gtk.Menu()

        item_show = Gtk.MenuItem(label="Upload Image")
        item_show.connect("activate", self.on_upload_image, (model, lv_iter, values))
        menu.append(item_show)

        item_show_json = Gtk.MenuItem(label="Show JSOn")
        item_show_json.connect("activate", self.on_show_json, (model, lv_iter, values))
        menu.append(item_show_json)

        item_show_com = Gtk.MenuItem(label="Edit Comments")
        item_show_com.connect("activate", self.on_show_comments, (model, lv_iter, values))
        menu.append(item_show_com)

        item_show_def = Gtk.MenuItem(label="Edit Defects")
        item_show_def.connect("activate", self.on_show_defects, (model, lv_iter, values))
        menu.append(item_show_def)

        menu.show_all()
        menu.popup_at_pointer(event)

    def on_upload_image(self, item, data):
        """Add defects with images.."""

        model, lv_iter, val = data

        irow = 0
        tree = Gtk.TreeView(model=val[self.F_LIST])
        tree.connect("button-press-event", self.on_file_pressed)

        btn = Gtk.Button(label="Add image")
        btn.set_tooltip_text("Click to add a new image.")
        btn.connect("clicked", self.on_add_image, val[PanelVisualInspection.F_LIST])

        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(tree)
        scrolled.set_size_request(-1, 150)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Description", renderer, text=PanelVisualInspection.F_DEFECT)
        tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("File", renderer, text=PanelVisualInspection.F_NAME)
        tree.append_column(column)

        dlg = Gtk.Dialog(title="Add Image")
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        box = dlg.get_content_area()
        box.add(btn)
        box.add(scrolled)
        dlg.show_all()
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            f_model = tree.get_model()
            n_files = f_model.iter_n_children()
            model.set_value(lv_iter, PanelVisualInspection.F_LIST, f_model)
            model.set_value(lv_iter, PanelVisualInspection.N_FILES, n_files)

        dlg.hide()
        dlg.destroy()

        self.write_message("Defects added\n")

    def on_file_pressed(self, tree, event):
        """Called when right button clicked in add image dialog.

        Opens a pop-up menu to delete the selected entry."""
        # double click shows attachments
        if event.button == 1 and event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            self.write_message("This is a double click.\n")
            return


        if event.button != 3:
            return

        model, lv_iter, values = self.get_iter_at_position(tree, event)
        if not values:
            return
        menu = Gtk.Menu()

        item_show = Gtk.MenuItem(label="Delete")
        item_show.connect("activate", self.on_delete_image, (model, lv_iter, values))
        menu.append(item_show)

        menu.show_all()
        menu.popup_at_pointer(event)

    def on_delete_image(self, item, data):
        """Delete a defect and image"""
        model, lv_iter, _ = data
        model.remove(lv_iter)

    def on_add_image(self, btn, model):
        """Adds a new image."""
        dlg = Gtk.Dialog(title="Add Image")

        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        box = dlg.get_content_area()
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        box.add(grid)

        irow = 0
        lbl = Gtk.Label(label="Description")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        desc = Gtk.Entry()
        grid.attach(desc, 1, irow, 1, 1)

        irow += 1
        lbl = Gtk.Label(label="Image")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        fC = Gtk.FileChooserButton()
        grid.attach(fC, 1, irow, 1, 1)

        dlg.show_all()

        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            D = desc.get_text()
            P = Path(fC.get_filename()).expanduser().resolve()
            model.append([D, P.name, P.as_posix()])

        dlg.hide()
        dlg.destroy()

    def on_show_json(self, item, data):
        """Test JSon."""
        model, lv_iter, val = data
        payload = val[PanelVisualInspection.TEST_J].js
        value, dlg = create_json_data_editor(payload)
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            payload = value.values
            model.set_value(lv_iter, PanelVisualInspection.TEST_J, TestJson(payload))

        dlg.hide()
        dlg.destroy()

    def on_show_comments(self, item, data):
        """Show comments"""
        model, lv_iter, val = data
        js = val[PanelVisualInspection.TEST_J].js
        SC = ShowComments("Test Comments", js["comments"], self)
        rc = SC.run()
        if rc == Gtk.ResponseType.OK:
            js["comments"] = SC.comments
            model.set_value(lv_iter, PanelVisualInspection.TEST_J, TestJson(js))

        SC.hide()
        SC.destroy()

    def on_show_defects(self, item, data):
        """Show comments"""
        model, lv_iter, val = data
        js = val[PanelVisualInspection.TEST_J].js
        SD = ShowDefects("Test Defects", js["defects"], self)
        rc = SD.run()
        if rc == Gtk.ResponseType.OK:
            js["defects"] = SD.defects
            model.set_value(lv_iter, PanelVisualInspection.TEST_J, TestJson(js))

        SD.hide()
        SD.destroy()

    def SN_ready(self, *args):
        """SN is ready in the TextEnttry."""
        SN = self.SN.get_text()
        # GEt children.
        panel = ITkDButils.get_DB_component(self.session, SN)
        if panel is None:
            self.write_message(ITkDButils.get_db_response())
            return

        SN = panel["serialNumber"]
        args[0].set_text(SN)
        is_PWB = False
        defaults = {
            "institution": self.institute,
            "runNumber": "1",
            "date": self.date.get_text()
        }
        component_type = None
        test_code = None
        if "USED" in SN:
            # Powerboard Carrier
            if not SN[6].isdigit():
                dbGtkUtils.complain("Not a Powerboard Carrier",
                                    "{}: wrong SN for a powerboard carrier".format(SN))
                self.SN.widget.set_text("")
                return

            self.panel_type.set_text("PWB carrier")
            is_PWB = True
            component_type = "PWB"
            test_code = "PICTURE"

        elif "USET" in SN:
            # Hybrid test panel
            component_type = "HYBRID_TEST_PANEL"
            test_code = "VISUAL_INSPECTION"

            if not SN[6].isdigit or int(SN[6])>5:
                dbGtkUtils.complain("Not a Hybrid Test Panel",
                                    "{}: wrong SN for a hybrid test panel".format(SN))
                self.SN.widget.set_text("")
                return

            self.panel_type.set_text("HYB test panel")

        else:
            dbGtkUtils.complain("Invalid SN.",
                "{}\nNot a PWB carrier not HYB test panel.".format(SN))
            self.SN.widget.set_text("")
            return

        # GEt children.
        skltn = ITkDButils.get_test_skeleton(self.session, component_type, test_code, defaults)
        model = self.create_model()
        for child in panel["children"]:
            if child["component"] is not None:
                child_SN = child["component"]["serialNumber"]
                skltn["component"] = child_SN
                if is_PWB:
                    position = child["order"]
                else:
                    position = -1
                    for P in child["properties"]:
                        if P["code"] == "HYBRID_POSITION":
                            if P["value"] is not None:
                                position = int(P["value"])
                                break

                model.append([child_SN, position, True, 0, self.create_file_model(), TestJson(skltn)])

        self.tree.set_model(model)


    def upload_tests(self, *args):
        """Upload the current test."""
        SN = self.SN.get_text()

        model = self.tree.get_model()
        lv_iter = model.get_iter_first()
        n_items = 0
        global_link = None
        while lv_iter:
            values = model[lv_iter]
            payload = values[PanelVisualInspection.TEST_J].js

            attachments = []
            if global_link is None:
                if self.global_image is not None:
                    A = ITkDButils.Attachment(path=self.global_image, title="Global Image", desc="Image of whole panel")
                    attachments.append(A)

            im_model = values[PanelVisualInspection.F_LIST]
            im_iter = im_model.get_iter_first()
            idef = 1
            while im_iter:
                defect, name, path = im_model[im_iter]
                A = ITkDButils.Attachment(path=path, title="Defect {}".format(idef), desc=defect)
                attachments.append(A)
                idef += 1
                im_iter = im_model.iter_next(im_iter)

            rc = ITkDButils.upload_test(self.session, payload, attachments, check_runNumber=True)
            if rc:
                ipos = rc.find("The following details may help:")
                msg = rc[ipos:]
                dbGtkUtils.complain("Failed uploading test {}-{}".format(payload["component"], payload["testType"]), msg)
                self.write_message(msg+"\n")

            else:
                self.write_message("Upload {}-{} successfull\n".format(payload["component"], payload["testType"]))
                if global_link is None:
                    try:
                        if self.global_image is not None:
                          	global_link = ITkDButils.attachment_urls[self.global_image.name]
                    except KeyError:
                        pass

                if payload["testType"] == "PICTURE":
                    rc = ITkDButils.set_test_run_parameter(self.session,
                                                           ITkDButils.uploaded_test_runs[0],
                                                           "COMMENT", "Picture of PW carrier")

                    rc = ITkDButils.set_test_run_parameter(self.session,
                                                           ITkDButils.uploaded_test_runs[0],
                                                           "LINK_TO_PICTURE", global_link)
                    if rc:
                        ipos = rc.find("The following details may help:")
                        msg = rc[ipos:]
                        dbGtkUtils.complain("Failed updating LINK_TO_PICTURE {}-{}".format(payload["component"],
                                                                                           payload["testType"]), msg)
                        self.write_message(msg+'\n')

                elif payload["testType"] == "VISUAL_INSPECTION":
                    rc = ITkDButils.create_test_run_comment(self.session,
                                                            ITkDButils.uploaded_test_runs[0],
                                                            ["Link to global image:\n {}".format(global_link)])
                    if rc:
                        ipos = rc.find("The following details may help:")
                        msg = rc[ipos:]
                        dbGtkUtils.complain("Failed adding global link as comment: {}-{}".format(payload["component"],
                                                                                                 payload["testType"]), msg)
                        self.write_message(msg+"\n")
                    else:
                        self.write_message("Image: {}\n".format(global_link))


            n_items += 1
            lv_iter = model.iter_next(lv_iter)


    def get_qrcode(self,txt):
        """Read SN from scanner."""
        self.write_message("SN: {}\n".format(txt))
        self.SN_ready(txt, self.SN.widget)


def main():
    """Main entry."""
    # DB login
    HELP_LINK="https://itkdb-gtk.docs.cern.ch/panelVisualInspection.html"

    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    gTest = PanelVisualInspection(client, help_link=HELP_LINK)

    gTest.present()
    gTest.connect("destroy", Gtk.main_quit)
    try:
        Gtk.main()

    except KeyboardInterrupt:
        print("Arrrgggg!!!")

    dlg.die()

if __name__ == "__main__":
    main()
